## SCRIPT INITIATION ##########################################################
## set working directory
setwd(dir = "~/Documents/projects/Articles/EMMAgeo/")

## define colours
col_1 <- c("darkblue", "orange3", "darkgreen", "violetred4")

col_2 <- colorRampPalette(colors = c("darkblue", "blue", "green",
                                     "grey", 
                                     "yellow", "orange", "red"))

col_3 <- c("orange", "red", "brown", "grey30", "slateblue4", "springgreen4")

## load data sets
load(file = "data/benchmark.rda")

## TABLE for X with R and E statistics ########################################
S_X <- do.call(rbind, lapply(X = B, FUN = function(B) {
  
  Rt <- cor(x = as.numeric(B$X_in), 
            y = as.numeric(B$X))^2
  
  Rm <- mean(sapply(X = 1:nrow(B$X_in), FUN = function(i) {
    cor(B$X_in[i,], B$X[i,])^2
  } ))
  
  Rn <- mean(sapply(X = 1:ncol(B$X_in), FUN = function(i) {
    r <- cor(B$X_in[,i], B$X[,i])^2
    ifelse(is.na(r) == TRUE, 0, r)
  } ))
  
  # Et <- sqrt(mean((as.numeric(B$X_in) - as.numeric(B$X))^2))
  
  # Em <- sqrt(mean(sapply(X = 1:nrow(B$X_in), FUN = function(i) {
  #   (B$X_in[i,] - B$X[i,])^2
  # } )))
  
  # En <- sqrt(mean(sapply(X = 1:ncol(B$X_in), FUN = function(i) {
  #   (B$X_in[,i] - B$X[,i])^2
  # } )))
  
  Em_raw <- numeric(length = nrow(B$X_in))
  for(i in 1:length(Em_raw)) {
    
    Em_raw[i] <- sqrt(mean((B$X_in[i,] - B$X[i,])^2))
  }
  Em <- mean(Em_raw)
  
  En_raw <- numeric(length = ncol(B$X_in))
  for(i in 1:length(En_raw)) {
    
    En_raw[i] <- sqrt(mean((B$X_in[,i] - B$X[,i])^2))
  }
  En <- mean(En_raw)
  
  Et <- mean(c(Em, En))
  
  return(data.frame(Rt, Rm, Rn, Et, Em, En))
}))

S_X <- round(S_X, 3)


## TABLE for EM with R and E statistics ########################################
S_EM <- lapply(X = B, FUN = function(B) {
  
  EM_nat <- as.matrix(B$l_in)
  EM_mod <- B$l
  
  EM_ok <- ifelse(test = EM_nat > 0.1, 
                  yes = 1, 
                  no = NA)
  
  EM_nat <- EM_nat * EM_ok
  EM_mod <- EM_mod * EM_ok
  
  EM_nat <- EM_nat /rowSums(EM_nat, na.rm = TRUE) * 100
  EM_mod <- EM_mod /rowSums(EM_mod, na.rm = TRUE) * 100
  
  Rm <- sapply(X = 1:nrow(EM_nat), FUN = function(i) {
    cor(EM_nat[i,], EM_mod[i,], use = "pairwise.complete.obs")^2
  } )
  
  # Em <- (sapply(X = 1:nrow(EM_nat), FUN = function(i) {
  #   mean(sqrt((EM_nat[i,] - EM_mod[i,])^2), na.rm = TRUE)
  # } ))
  
  Em_raw <- numeric(length = nrow(EM_nat))
  for(i in 1:length(Em_raw)) {
    
    Em_raw[i] <- sqrt(mean((EM_nat[i,] - EM_mod[i,])^2, na.rm = TRUE))
  }
  Em <- (Em_raw)

  return(data.frame(Rm, Em))
})

S_EM <- do.call(rbind, lapply(X = S_EM, FUN = function(S_EM) {
  
  c(S_EM$Rm, S_EM$Em)
}))

colnames(S_EM) <- c(paste("Rm_", 1:4, sep = ""),
                 paste("Em_", 1:4, sep = ""))

S_EM <- S_EM[,c(4:1, 8:5)]

S_EM <- round(S_EM, 3)

## TABLE for ratios with R and E statistics ########################################
S_ratios <- lapply(X = B, FUN = function(B) {
  
  s_nat <- as.matrix(B$s_in)
  s_mod <- B$s
  
  Rn <- sapply(X = 1:ncol(s_nat), FUN = function(i) {
    cor(s_nat[,i], s_mod[,i], use = "pairwise.complete.obs")^2
  } )
  
  # En <- sapply(X = 1:ncol(s_nat), FUN = function(i) {
  #   mean(sqrt((s_nat[,i] - s_mod[,i])^2), na.rm = TRUE)
  # } )
  
  En_raw <- numeric(length = ncol(s_nat))
  for(i in 1:length(En_raw)) {
    
    En_raw[i] <- sqrt(mean((s_nat[,i] - s_mod[,i])^2, na.rm = TRUE))
  }
  En <- (En_raw)
  
  return(data.frame(Rn, En))
})

S_ratios <- do.call(rbind, lapply(X = S_ratios, FUN = function(S_ratios) {
  
  c(S_ratios$Rn, S_ratios$En)
}))

colnames(S_ratios) <- c(paste("Rn_", 1:4, sep = ""),
                    paste("En_", 1:4, sep = ""))

S_ratios <- S_ratios[,c(4:1, 8:5)]

S_ratios <- round(S_ratios, 3)


S_all <- cbind(S_X, S_EM, S_ratios)
S_all_R <- S_all[,grepl(x = colnames(S_all), pattern = "R")]
S_all_E <- S_all[,grepl(x = colnames(S_all), pattern = "E")]

## TABLE for mode and Mqs statistics ##########################################
S_mode <- lapply(X = B, FUN = function(B) {
  
  EM_nat <- as.matrix(B$l_in)
  EM_mod <- B$l
  
  mu <- B$mu
  mu <- seq(from = 1, to = ncol(EM_nat))

  mode_nat <- sapply(X = 1:nrow(EM_nat), FUN = function(i) {
    
    mu[EM_nat[i,] == max(EM_nat[i,])]
  })

  mode_mod <- sapply(X = 1:nrow(EM_nat), FUN = function(i) {
    
    mu[EM_mod[i,] == max(EM_mod[i,])]
  })  
  
  return(mode_mod - mode_nat)
})

S_mode <- do.call(rbind, S_mode)

colnames(S_mode) <- paste("dmode_", 1:4, sep = "")

S_mode <- S_mode[,c(4:1)]

S_mode <- round(S_mode, 1)

## PLOT all grain-size curves #################################################
# pdf(file = "00_third_attempt/EM_nat_and_curves.pdf", 
#     width = 7, 
#     height = 5)

jpeg(file = "00_third_attempt/EM_nat_and_curves.jpg", 
     width = 2000, 
     height = 1700,
     res = 300)

plot(NA,
     xlim = range(mu), 
     ylim = range(EM_nat), 
     log = "x",
     xlab = "Grain-size (um)",
     ylab = "Vol.-%")

for(i in 1:nrow(X_nat)) {
  lines(x = mu, 
        y = X_nat[i,],
        col = adjustcolor(col = 1, alpha.f = 0.3))
}

for(i in 1:nrow(EM_nat)) {
  lines(x = mu, 
        y = EM_nat[i,],
        col = rev(col_1)[i],
        lwd = 2)
}

dev.off()

## PLOT images of difference ##################################################
# pdf(file = "00_third_attempt/figure_images_of_difference.pdf", 
#     width = 14, 
#     height = 11)

jpeg(filename = "00_third_attempt/figure_images_of_difference.jpeg", 
    width = 3000, 
    height = 2000, 
    res = 300)

par(mfcol = c(2, 3), 
    mar = c(5, 4, 2, 5))

lapply(X = B, FUN = function(B) {
  
  D <- as.matrix(B$X - B$X_in)
  
  fields::image.plot(x = B$mu, 
                     y = 1:nrow(D),
                     z = t(D), 
                     log = "x", 
                     zlim = c(-3, 3), 
                     col = col_2(200),
                     xlab = "Grain-size (um)",
                     ylab = "Sample ID",
                     legend.lab = "Devation (Vol.-%)",
                     legend.line = 2.5,
                     main = B$meta$name)
  
})

dev.off()


## PLOT Visualisation of all R criteria #######################################
# pdf(file = "00_third_attempt/R_summary.pdf", 
#     width = 7, 
#     height = 5)

# jpeg(file = "00_third_attempt/R_summary.jpg", 
#      width = 2000, 
#      height = 1700,
#      res = 300)
# 
# plot(NA, 
#      xlim = c(1, ncol(S_all_R) + 5), 
#      ylim = range(S_all_R), 
#      main = "Model synthesis - R", 
#      xlab = "Criterion",
#      ylab = expression(R^2),
#      axes = FALSE)
# axis(side = 1, 
#      at = 1:ncol(S_all_R), 
#      labels = colnames(S_all_R))
# axis(side = 2)
# box(which = "plot")
# 
# legend(x = "right", 
#        legend = rownames(S_all_R), 
#        col = col_3, 
#        lty = 1, 
#        pch = 1)
# 
# for(i in 1:nrow(S_all_R)) {
#   
#   points(x = 1:ncol(S_all_R),
#          y = S_all_R[i,], 
#          col = col_3[i])
#   
#   lines(x = 1:3,
#          y = S_all_R[i,1:3], 
#          col = col_3[i])
#   
#   lines(x = 4:7,
#         y = S_all_R[i,4:7], 
#         col = col_3[i])
# 
#   lines(x = 8:11,
#         y = S_all_R[i,8:11], 
#         col = col_3[i])
# }
# 
# dev.off()

## PLOT Visualisation of all E criteria #######################################
# pdf(file = "00_third_attempt/E_summary.pdf", 
#     width = 7, 
#     height = 5)

# jpeg(file = "00_third_attempt/E_summary.jpg", 
#     width = 2000, 
#     height = 1700,
#     res = 300)
# 
# plot(NA, 
#      xlim = c(1, ncol(S_all_E) + 5), 
#      ylim = range(S_all_E), 
#      main = "Model synthesis - R", 
#      xlab = "Criterion",
#      ylab = "Root mean square error",
#      axes = FALSE)
# axis(side = 1, 
#      at = 1:ncol(S_all_E), 
#      labels = colnames(S_all_E))
# axis(side = 2)
# box(which = "plot")
# 
# legend(x = "right", 
#        legend = rownames(S_all_E), 
#        col = col_3, 
#        lty = 1, 
#        pch = 1)
# 
# for(i in 1:nrow(S_all_E)) {
#   
#   points(x = 1:ncol(S_all_E),
#          y = S_all_E[i,], 
#          col = col_3[i])
#   
#   lines(x = 1:3,
#         y = S_all_E[i,1:3], 
#         col = col_3[i])
#   
#   lines(x = 4:7,
#         y = S_all_E[i,4:7], 
#         col = col_3[i])
#   
#   lines(x = 8:11,
#         y = S_all_E[i,8:11], 
#         col = col_3[i])
#   
# }
# 
# dev.off()

## PLOT loadings vs. EM_nat ###################################################
# pdf(file = "00_third_attempt/comparison_loadings.pdf", 
#     width = 7, 
#     height = 5)

# jpeg(file = "00_third_attempt/comparison_loadings.jpg", 
#      width = 3000, 
#      height = 2000,
#      res = 300)

pdf(file = "00_third_attempt/comparison_loadings.pdf", 
    width = 3000 / 300, 
    height = 2000 / 300)

par(mfcol = c(2, 3))

B_plot <- B[c(1, 4, 2, 5, 3, 6)]

lapply(X = B_plot, FUN = function(B) {
  
  plot(NA, 
       xlim = c(0, 10),
       ylim = c(0, 10),
       main = B$meta$name,
       xlab = "Vol.-%",
       ylab = "Vol.-%")
  
  abline(a = 0, 
         b = 1, 
         col = "grey30", 
         lwd = 2)
  
  for(i in 1:nrow(B$l_in)) {
    
    points(x = B$l_in[i,],
           y = B$l[i,],
           col = rev(col_1)[i])
    lines(x = B$l_in[i,],
           y = B$l[i,],
           col = rev(col_1)[i])
  }
  
})

dev.off()

## PLOT scores vs. ratios ###################################################
# pdf(file = "00_third_attempt/comparison_loadings.pdf", 
#     width = 7, 
#     height = 5)

# jpeg(filename = "00_third_attempt/comparison_scores.jpg", 
#      width = 3000, 
#      height = 2000,
#      res = 300)

pdf(file = "00_third_attempt/comparison_scores.pdf", 
     width = 3000 / 300, 
     height = 2000 / 300)

par(mfcol = c(2, 3))

B_plot <- B[c(1, 4, 2, 5, 3, 6)]

lapply(X = B_plot, FUN = function(B) {
  
  plot(NA, 
       xlim = c(0, 1),
       ylim = c(0, 1),
       main = B$meta$name,
       xlab = "Vol.-%",
       ylab = "Vol.-%")
  
  abline(a = 0, 
         b = 1, 
         col = "grey30", 
         lwd = 2)
  
  for(i in 1:ncol(B$s_in)) {
    
    points(x = B$s_in[,i],
           y = B$s[,i],
           col = rev(col_1)[i])
    
  }
  
})

dev.off()

# write.table(x = S_X, 
#             file = "00_third_attempt/table_statistics_X.txt", 
#             sep = "\t")
# write.table(x = S_EM, 
#             file = "00_third_attempt/table_statistics_EM.txt", 
#             sep = "\t")
# write.table(x = S_ratios, 
#             file = "00_third_attempt/table_statistics_ratios.txt", 
#             sep = "\t")
# write.table(x = S_mode, 
#             file = "00_third_attempt/table_statistics_modes.txt", 
#             sep = "\t")
